@@ -1,7 +1,7 @@
 #Simulation of data sets
 #with a hierarchical design
 
-sim.data=function(nb.pept=10000,nb.miss=6000,pi.mcar=0.2,para=1.5,nb.cond=1,nb.repbio=3,nb.sample=3,m.c=25,sd.c=2,sd.rb=0.5,sd.r=0.2){
+sim.data=function(nb.pept=15000,nb.miss=5000,pi.mcar=0.2,para=3,nb.cond=1,nb.repbio=3,nb.sample=3,m.c=25,sd.c=2,sd.rb=0.5,sd.r=0.2){
 
   tab=matrix(0,nb.pept,nb.sample*nb.cond*nb.repbio);
   pmnar=tab;
